/************************************************************************
*
*		   TCA Energy Range Checking Program
*			Product # 48-0913
*
*	This program is used to check the energy range of the individual
*	centroid/width values for a given record number.  The program
*	uses SUCINT1 as the record number, and then compares the following
*	real values against SURINT2 (for the given record):
*
*	SURREAL1
*	SURREAL2
*	SURREAL3
*	SURREAL4
*	SURREAL5
*	SURREAL6
*
*	A local symbol is set to indicate that the results are GOOD, or
*	BAD.  The symbol name is ENERGYCHECK and it is a string. 
*
*	Calling Format:
*
*	ND_TCA_CHECKENERGY <config>
*
*	Where:
*		<config> is the input configuration setup
*
*************************************************************************
*
*	Revision History:
*
*	07-DEC-1996	ARW	Original version
*
************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <descrip.h>
#include <ndtypes.h>
#include <camdef.h>
#include <campardef.h>
#include <cammisc.h>
#include <cammsgdef.h>
#include <starlet.h>
#include <lib$routines.h>
#include <libclidef.h>

/* error handling macro */
#define IFERROR(status) if(!(status&1))

/* local functions */
struct dsc$descriptor_s createDescrip(char *s);
int setSymbol(char *name, char *value);

/* external functions */
extern cam_open(); 
extern cam_close(); 
extern cam_getp_l(); 

/************************************************************************/
main(int argc, char *argv[])
{

LONG s;          			/* return status */
LONG chan; 				/* channel number */
LONG flags; 				/* cam flags */
LONG record; 				/* record number */
LONG commonrec = 1; 			/* common record number */
LONG surint2;
REAL surreal1; 
REAL surreal2; 
REAL surreal3; 
REAL surreal4; 
REAL surreal5; 
REAL surreal6; 
struct dsc$descriptor_s camfile;	/* cam file descriptor */

/* the special uses record number */
struct cam_plist_struct plist1[] = {
        { CAM_L_SUCINT1,        0, NULL },
        { 0,                    0, NULL }
};

/* the list of items to retrieve for the record */
struct cam_plist_struct plist2[] = {
        { CAM_F_SURREAL1,        0, NULL },
        { CAM_F_SURREAL2,        0, NULL },
        { CAM_F_SURREAL3,        0, NULL },
        { CAM_F_SURREAL4,        0, NULL },
        { CAM_F_SURREAL5,        0, NULL },
        { CAM_F_SURREAL6,        0, NULL },
        { CAM_L_SURINT2,         0, NULL },
        { 0,                     0, NULL }
};

plist1[0].address = &record;		/* record number */

plist2[0].address = &surreal1;
plist2[1].address = &surreal2;
plist2[2].address = &surreal3;
plist2[3].address = &surreal4;
plist2[4].address = &surreal5;
plist2[5].address = &surreal6;
plist2[6].address = &surint2;

	/* make sure the program was called correctly */
	if (argc < 2)
	{
		fprintf(stdout, "Usage: ND_TCA_CHECKENERGY <config>\n"); 
		exit(1); 
	}

	/* create a descriptor with the cam file name */
	camfile = createDescrip(argv[1]); 

	/* open the cam file with the spectrum information */
	flags = CAM_M_COC_RONLY; 
	s = cam_open(&chan, &camfile, &flags); 
	IFERROR(s) exit(s); 

	/* get the record number */
        flags = 0; 
        s = cam_getp_l(&chan, plist1, &commonrec, 0, &flags);
        IFERROR(s) exit(s); 

	/* get all of the values to compare */
        flags = 0; 
        s = cam_getp_l(&chan, plist2, &record, 0, &flags);
        IFERROR(s) exit(s); 

	/* close the input cam file */
	s = cam_close(&chan); 
	IFERROR(s) exit(s); 
	
	/* perform the test to see if the numbers are within range */
	if ((surreal1 > surint2) || (surreal2 > surint2) || (surreal3 > surint2) ||
		(surreal4 > surint2) || (surreal5 > surint2) || (surreal6 > surint2))
	{
		s = setSymbol("ENERGYCHECK", "BAD"); 
        	IFERROR(s) exit(s); 
	}
	else 
	{
		s = setSymbol("ENERGYCHECK", "GOOD"); 
        	IFERROR(s) exit(s); 
	}

	/* return success */
	return(1); 
} 

/************************************************************************
*
* Function: createDescrip
*
* Description: 	This function will return a string descriptor properly 
* 		initialized to point to the null terminated string that
*		is passed in as an argument. 
*
* Usage:
*
*	descrip = createDescrip(string)
*
*	where:
*
*		descrip = returned string descriptor structure
*       	string  = a character pointer to a null terminated string
*	
************************************************************************/

struct dsc$descriptor_s createDescrip(char *s)
{

struct dsc$descriptor_s temp; 

	temp.dsc$b_class = DSC$K_CLASS_S; 	/* string descriptor */
	temp.dsc$b_dtype  = DSC$K_DTYPE_T; 	/* text data type */
	temp.dsc$a_pointer = s; 		/* string address */
	temp.dsc$w_length = strlen(s); 		/* length of string */

	/* return the initialized string descriptor */
	return(temp); 

}

/************************************************************************
*
* Function: setSymbol
*
* Description: 	This function will set an individual DCL symbol. 
*
* Usage:
*
*	s = setSymbol(name, value)
*
*	where:
*
*		s = return status
*
*		name = symbol name to set
*
*		value = string to set the symbol
*	
************************************************************************/

int setSymbol(char *name, char *value)
{

struct dsc$descriptor_s temp1; 
struct dsc$descriptor_s temp2; 
int table = LIB$K_CLI_LOCAL_SYM; 


	/* create descriptors for the name and value */
	temp1 = createDescrip(name); 
	temp2 = createDescrip(value); 

	/* call the function to set up the global symbol */
	return(lib$set_symbol(&temp1, &temp2, &table)); 

}

/* end of file */

